import os
import pickle
import sys
import traceback
import xbmcgui
import xbmc
import youtube
import random
import string
import re

class YouTubeGUI(xbmcgui.WindowXML):
    
	# GUI States.
	STATE_MAIN             = 1
	STATE_FEEDS            = 2
	STATE_USERS            = 4
	
	STATE_MOST_DISCUSSED   = STATE_FEEDS | 8
	STATE_MOST_VIEWED      = STATE_MOST_DISCUSSED | 16
	STATE_MOST_LINKED      = STATE_MOST_VIEWED    | 32
	STATE_MOST_POPULAR     = STATE_MOST_LINKED    | 64
	STATE_TOP_FAVORITES    = STATE_MOST_POPULAR   | 128
	STATE_TOP_RATED        = STATE_TOP_FAVORITES  | 256
	
	STATE_CATEGORIES       = 512
	STATE_SEARCH           = 1024
	STATE_USERS            = 2048

	# Content list states
	CONTENT_STATE_NONE     = 0
	CONTENT_STATE_VIDEO    = 1
	CONTENT_STATE_USERS    = 2
	CONTENT_STATE_FAVORITES = 4
	CONTENT_STATE_SEARCH_HISTORY = 8
	

	def onInit(self):
    	
		
		try:
			if (self.initialized == True):
									
				self.update_list_mini(self.getList())				
				pass
				
		except:
			try:
			
				#list to store all the list. enables the 'back' feature in context menu.
				self.content_lists = []
				self.content_lists_position = -1

				#shouldn't be here. move to def update_list
				self.enableDlg = True			
				
				self.init_button_state()
				self.initialized = True
				self.base_path = os.getcwd().replace(';','')

				self.data = []
				self.current_position = 0
				
				self.yt = youtube.YouTube()
				self.yt.set_filter_hook(self.confirm_content)

				self.settings = self.get_settings()
				self.update_youtube_quality()
				
				self.state = YouTubeGUI.STATE_MAIN
				self.list_state = YouTubeGUI.CONTENT_STATE_NONE


				# Get the last search term
				history = self.get_search_history()
				if history is not None and len(history) > 0:
					self.last_search_term = history[0]
				else:
					self.last_search_term = ''

				self.player = xbmc.Player(xbmc.PLAYER_CORE_MPLAYER)
				self.getList().addItem('')
				self.get_feed('recently_featured')
								

			except:
				xbmc.log('Exception (init): ' + str(sys.exc_info()[0]))
				traceback.print_exc()
				self.close()  
			
		
		# Put your List Populating code/ and GUI startup stuff here
		pass 
 
 
 	def onAction(self, action):
 		"""Handle user input events."""
  
 		try: 
 			# BACK BUTTON
 			if action.getId() == 10:
 				if self.state is YouTubeGUI.STATE_MAIN:
 					self.close()
 				elif self.state & ~YouTubeGUI.STATE_FEEDS & YouTubeGUI.STATE_MOST_DISCUSSED:
 					self.set_button_state(YouTubeGUI.STATE_FEEDS)
 				else:
 					self.set_button_state(YouTubeGUI.STATE_MAIN)

			# CONTEXT MENU
 			elif action.getId() == 117:
 				if self.list_state is YouTubeGUI.CONTENT_STATE_VIDEO:
 					self.context_menu_video()
 				else:
 					self.not_implemented()
 			
 		except:
 			xbmc.log('Exception (onAction): ' + str(sys.exc_info()[0]))
 			traceback.print_exc()
 			self.close()
 
 	def onClick(self, controlID):
 		"""Handle widget events."""
 
 		ctrl = self.getControl(controlID)
 		 		
 		try:  			
 			if controlID >= 50 and controlID < 60:
 				self.on_control_video_list(self) 			 				
 				
 			elif controlID == 130:
 				self.get_next()
 				
 			elif controlID == 131:
 				self.get_prev() 		
 				
 			elif controlID == 15:
 				self.settings_window()
 				
 			elif self.state is YouTubeGUI.STATE_MAIN:
 				self.on_control_main(ctrl)
 				
 			elif self.state is YouTubeGUI.STATE_SEARCH:
 				self.on_control_search(ctrl)
 				
 			elif self.state is YouTubeGUI.STATE_USERS:
 				self.on_control_users(ctrl)
 				
 			elif self.state is YouTubeGUI.STATE_FEEDS:
 				self.on_control_feeds(ctrl)
 				
 			elif self.state is YouTubeGUI.STATE_MOST_VIEWED:
 				self.get_feed_by_time(ctrl, "most_viewed")
 				
 			elif self.state is YouTubeGUI.STATE_MOST_DISCUSSED:
 				self.get_feed_by_time(ctrl, "most_discussed")
 				
 			elif self.state is YouTubeGUI.STATE_MOST_LINKED:
 				self.get_feed_by_time(ctrl, "most_linked")
 				
 			elif self.state is YouTubeGUI.STATE_MOST_POPULAR:
 				self.get_feed_by_time(ctrl, "most_popular")
 				
 			elif self.state is YouTubeGUI.STATE_TOP_FAVORITES:
 				self.get_feed_by_time(ctrl, "top_favorites")
 				
 			elif self.state is YouTubeGUI.STATE_TOP_RATED:
 				self.get_feed_by_time(ctrl, "top_rated")
 				
 				
 				
 		except:
 			xbmc.log('Exception (onControl): ' + str(sys.exc_info()[0]))
 			traceback.print_exc()
 			self.close()
			
			
			

	def onFocus(self, controlID):
		
		pass
	        

	def confirm_content(self, udata):
		"""Let the user choose to watch offensive crap."""

		dlg = xbmcgui.Dialog()
		return dlg.yesno('YouTube',
		                 'This clip may contain inappropriate content.',
		                 'Do you want to watch it anyway?')


	def settings_window(self):
	
		
		itms = ["High Quality", "Thumbnails", "Default User Favorites", "Default User Videos", "Default User Playlists", "Delete Thumbs in Cache"]		

		result = 0		
		
		while(result >= 0 and result < len(itms)):
			tmpitems = []
			counter = 0
						
			for itm in itms:
			
				#last item is "delete thumbcache", don't change it.
				if itm == itms[len(itms)-1]:
					tmpitems.append(str(itm) + ": " + str(self.get_number_thumbs()))
				else:					
					tmp = itm + ": "
					if self.settings[counter] == 1 and counter < 2:
						tmp += "Yes"
					elif counter >= 2:
						tmp += `self.settings[counter]`
					else:
						tmp += "No"
					tmpitems.append(tmp)

				counter += 1
				
			tmpitems.append("Done")
			
			result = xbmcgui.Dialog().select("Toggle Settings:", tmpitems)	
			
			if (result >= 0 and result < len(itms)):
				if (result == 2 or result == 3 or result == 4):
					default = self.get_input('Default:', str(self.settings[result]))
					self.settings[result] = default	
				elif (result == 5):
					
					self.delete_thumb_cache()
					
				elif self.settings[result] == 1:
					self.settings[result] = 0
				else:
					self.settings[result] = 1
									
				self.save_settings()
				self.update_youtube_quality()
			
	def update_youtube_quality(self):
	
		if (self.settings[0] == 1):
			self.yt.setHighQuality()
		else:
			self.yt.setRegularQuality()
			
	
	def get_settings(self):
		"""Return a list of old search terms."""

		defaultsettings = [1,1,"user", "user", "user"]
		settings = []
		path = os.path.join(self.base_path, 'data', 'settings.txt')

		try:
			f = open(path, 'rb')
			settings = pickle.load(f)
			f.close()
			
			#if old settings linger around from previous version, reset them.
			if (len(settings) != len(defaultsettings)):
				settings = defaultsettings			
		except:
			# [high qual, thumbs, default user]
			settings = defaultsettings
			# File not found, will be created upon save
			pass		
		
		return settings

	def get_search_history(self):
		"""Return a list of old search terms."""

		history_list = []
		path = os.path.join(self.base_path, 'data', 'history.txt')

		try:
			f = open(path, 'rb')
			history_list = pickle.load(f)
			f.close()
		except:
			# File not found, will be created upon save
			pass

		return history_list

	def save_settings(self):
				
		try:
			dir = os.path.join(self.base_path, 'data')
			# Create the 'data' directory if it doesn't exist.
			if not os.path.exists(dir):
				os.mkdir(dir)
			path = os.path.join(dir, 'settings.txt')
			f = open(path, 'wb')
			pickle.dump(self.settings, f, protocol=pickle.HIGHEST_PROTOCOL)
			f.close()
		except IOError, e:
			print 'There was an error while saving the settings pickle (%s)' % e
		except:
			print "An unknown error occured during save settings\n"
			
	def add_search_history(self, term):
		"""Append a term, and trim the search history to max 50 entries."""

		history_list = self.get_search_history()

		# Filter out the term to add from the list so only one copy is stored.
		history_list = filter(lambda x: x != term, history_list)
		history_list.insert(0, term)

		if len(history_list) > 500:
			history_list = history_list[:500]

		try:
			dir = os.path.join(self.base_path, 'data')
			# Create the 'data' directory if it doesn't exist.
			if not os.path.exists(dir):
				os.mkdir(dir)
			path = os.path.join(dir, 'history.txt')
			f = open(path, 'wb')
			pickle.dump(history_list, f, protocol=pickle.HIGHEST_PROTOCOL)
			f.close()
		except IOError, e:
			print 'There was an error while saving the pickle (%s)' % e
		except:
			print "An unknown error occured\n"

	def progress_handler(self, done, total, dlg):
		"""Update progress dialog percent and return abort status."""

		percent = int((done * 100.0) / total)

		dlg.update(percent)
		return not dlg.iscanceled()
	
	def download_data(self, arg, func, save=False):
		"""Show a progress dialog while downloading and return the data."""

		if (save):
			self.func = func
			self.arg = arg
				
		
		dlg = xbmcgui.DialogProgress()
		dlg.create('YouTube', 'Downloading...')
			
		self.yt.set_report_hook(self.progress_handler, dlg)
		data = None

		try:		
			data = func(arg)
			
		except Exception, Message:
			#dlg.close()
			#err_dlg = xbmcgui.Dialog()
			#err_dlg.ok('YouTube', str(Message) + " " + str(func))
			data = None
		
		dlg.close()
		return data
		
	#
	# refreshes the list
	#
	def load_list(self):
		
		data = self.download_data(self.arg, self.func)

		if (data != None):
			lbl = self.getControl(5).getLabel()
			lbls = lbl.split('-')
			lbl = lbls[0] + "- Page " + self.yt.get_page()
			self.getControl(5).setLabel(lbl)
			
			self.update_list(data, self.getList())
			
			

	#
	# change the sort ordering for a particular result
	#
	def change_sort(self):
	
		items = self.yt.ordertypes
		result = xbmcgui.Dialog().select("Sort order:", ["Default", "Time Published", "Number of Views", "Rating"])
		if (result > -1):	
			self.yt.ordertype = result			
			self.load_list()


	#
	# next page button
	#
	def get_next(self):
	
		if (self.yt.do_next()):			
						
			self.load_list()

	#
	# prev page button
	#
	def get_prev(self):
	
		if (self.yt.do_prev()):
		
			self.load_list()
			
	
	def get_input(self, title, default="", hidden=False):
		"""Show a virtual keyboard and return the entered text."""
	
		ret = None
	
		keyboard = xbmc.Keyboard(default, title)
		keyboard.setHiddenInput(hidden)
		keyboard.doModal()
	
		if keyboard.isConfirmed():
			ret = keyboard.getText()
	
		return ret

	def get_user_playlists(self, method, title, user=None, default=""):
		"""Get rss data and update the list."""

		if user is None:
			user = self.get_input("Enter user for playlists: ", default)
			
		if user is not None:
			
			self.yt.reset_page()		
						
			data = self.download_data(user, method, True)
			
			dlg = xbmcgui.Dialog()			

			if (data != None):				
				titles = []
				ids = []
				for dat in data:
					titles.append(dat[0])
					ids.append(dat[1])
				
				if len(titles) != 0:
					ret = xbmcgui.Dialog().select("Select playlist", titles)
					if (ret >= 0 and ret < len(titles)):					
						myid = ids[ret]
						pattern = re.compile('playlist:(.+)')
						match = pattern.search(myid)
						plid = match.groups(1)
	
						self.get_playlist_feed(plid, titles[ret])
						
			
			if (data == None or len(titles) == 0):
				dlg = xbmcgui.Dialog()
				dlg.ok('YouTube', 'No playlists found.')								

	def get_user_videos(self, method, title, user=None, default=""):
		"""Get rss data and update the list."""

		if user is None:
			user = self.get_input("Enter user for videos: ", default)
			
		if user is not None:
			
			self.yt.reset_page()
			
			data = self.download_data(user, method, True)
			
			if (data != None):
				self.list_state = YouTubeGUI.CONTENT_STATE_VIDEO
				lbl = '%s user %s' % (title, user)
				self.getControl(5).setLabel(lbl + " - Page " + self.yt.get_page())
				self.update_list(data, self.getList())
				
		
	def get_youtube(self, lbl, arg, func):
	
		self.yt.reset_page()
			
		data = self.download_data(arg, func, True)
		if (data != None):
			self.getControl(5).setLabel(lbl + " - Page " + self.yt.get_page())
			self.update_list(data, self.getList())
			self.list_state = YouTubeGUI.CONTENT_STATE_VIDEO
	
	def go_back(self):

		mySize = len(self.content_lists)
		myPos = self.content_lists_position
		myPos -= 1
		if (myPos < 0):
			return		
		
		myData = self.content_lists[myPos][1]
		myTitle = self.content_lists[myPos][0]
		
		self.yt.thumbnail = self.content_lists[myPos][2]
		self.yt.author = self.content_lists[myPos][3]
		self.yt.views = self.content_lists[myPos][4]
		self.yt.rating = self.content_lists[myPos][5]
		self.yt.duration = self.content_lists[myPos][6]
		self.yt.related = self.content_lists[myPos][7]
		self.yt.responses = self.content_lists[myPos][8]
		
		self.update_list(myData, self.getList())
		self.getControl(5).setLabel(myTitle)
		self.list_state = YouTubeGUI.CONTENT_STATE_VIDEO		
		self.content_lists_position -= 1

	def go_forward(self):

		mySize = len(self.content_lists)
		myPos = self.content_lists_position
		myPos += 1
		if (myPos > mySize-1):
			return
			
		self.yt.thumbnail = self.content_lists[myPos][2]
		self.yt.author = self.content_lists[myPos][3]
		self.yt.views = self.content_lists[myPos][4]
		self.yt.rating = self.content_lists[myPos][5]
		self.yt.duration = self.content_lists[myPos][6]
		self.yt.related = self.content_lists[myPos][7]
		self.yt.responses = self.content_lists[myPos][8]
			
		self.update_list(self.content_lists[myPos][1], self.getList())
		self.getControl(5).setLabel(self.content_lists[myPos][0])
		self.list_state = YouTubeGUI.CONTENT_STATE_VIDEO		
		self.content_lists_position += 1
		
	def get_feed(self, feed):
		"""Get rss data and update the list."""
						
		self.yt.reset_page()
		
		if ("?" not in feed):
			feed += "?"
		
		data = self.download_data(feed, self.yt.get_feed, True)
		
		if (data != None):
			tmpfeed = feed.split('?')
			feed = tmpfeed[0]
			lbl = ' '.join(map(lambda x: x.capitalize(), feed.split('_')))
			myLabel = lbl + " - Page " + self.yt.get_page()
			self.getControl(5).setLabel(myLabel)
			
			self.update_list(data, self.getList())
			self.list_state = YouTubeGUI.CONTENT_STATE_VIDEO
			
			if (self.content_lists_position != len(self.content_lists)-1):				
				self.content_lists = self.content_lists[:self.content_lists_position+1]
				
			myData = [myLabel, data, self.yt.thumbnail, self.yt.author, self.yt.views, self.yt.rating, self.yt.duration, self.yt.related, self.yt.responses]
			self.content_lists.append(myData)
			self.content_lists_position += 1
			
	def get_playlist_feed(self, feed, title):
		"""Get rss data and update the list."""		
		
		self.yt.reset_page()
		
		data = self.download_data(feed, self.yt.get_playlist_feed, True)
		
		if (data != None):			
			lbl = title
			self.getControl(5).setLabel(lbl + " - Page " + self.yt.get_page())
			
			self.update_list(data, self.getList())
			self.list_state = YouTubeGUI.CONTENT_STATE_VIDEO
					

	def get_cat(self, cstr):
		"""Get rss data and update the list."""
						
		self.yt.reset_page()
		
		data = self.download_data(cstr, self.yt.get_category, True)
		
		if (data != None):
			tmpfeed = cstr			
			lbl = ' '.join(map(lambda x: x.capitalize(), tmpfeed.split('_')))
			self.getControl(5).setLabel(lbl + " - Page " + self.yt.get_page())		
			if self.update_list(data, self.getList()):			
				self.list_state = YouTubeGUI.CONTENT_STATE_VIDEO


		
	def search(self, term=None):
		"""Get user input and perform a search. On success update the list."""

		if term is None:
			term = self.get_input('Search', self.last_search_term)					
		
		# Only update the list if the user entered something.
		if term != None:
		
			p = re.compile('[^a-zA-Z0-9 ]')
			term = p.sub('', term)
		
			self.yt.reset_page()
			
			self.last_search_term = term
			data = self.download_data(term, self.yt.search, True)
				
			if (data != None):
				self.add_search_history(term)
				self.list_state = YouTubeGUI.CONTENT_STATE_VIDEO
				lbl = 'Search: %s' % term
				self.getControl(5).setLabel(lbl + " - Page " + self.yt.get_page())
				self.update_list(data, self.getList())
				
			else:
				dlg = xbmcgui.Dialog()
				dlg.ok('YouTube', 'No videos were found that match your query.')
				self.setFocus(self.getControl(40))
				
		
		
	def search_history(self):
		"""Get search history and update the list."""
		
		data = [x for x in self.get_search_history()]
		
		if (len(data) <= 0):		
			dlg = xbmcgui.Dialog()
			dlg.ok('YouTube', 'Search history empty.')
			return
		else:
			ret = xbmcgui.Dialog().select("Search history", data)
			if (ret >= 0 and ret < len(data)):
				self.search(data[ret])

	#
	# "wrapper" function in case we use control lists, or window lists.	
	#
	def listClear(self, content_list):
	
		content_list.clearList()
	
	
	#
	# updates the list for onInit.
	#
	def update_list_mini(self, content_list):
		
		xbmcgui.lock()
		
		self.listClear(content_list)
		
		for listitem in self.listItems:
			content_list.addItem(listitem)		
		
		content_list.setCurrentListPosition(self.current_position)
		
		xbmcgui.unlock()
	
	#
	# gets number of thumbs
	#
	def get_number_thumbs(self):
		dir = os.path.join(self.base_path, 'data')
		files = os.listdir(dir)
		ret = len(files) - 3
		if (ret < 0):
			ret = 0
		return ret
		
	#
	# deletes thumbs in our 'cache'
	#
	def delete_thumb_cache(self):
	
		dlg = xbmcgui.DialogProgress()
		dlg.create('YouTube', 'Deleting thumbs...')
			
		dir = os.path.join(self.base_path, 'data')
		files = os.listdir(dir)
		
		totalsize = len(files)
		counter = 0
		
		for file in files:
		
			percent = int((counter * 100.0) / totalsize)
			dlg.update(percent)
			
			if file == '.' or file == '..': continue
			try:
				path = dir + os.sep + file
				f = file.split(".")
				#only remove files ending in jpg
				if (f[1]=="jpg"):
					os.remove(path)
			except:
				pass
			
			counter += 1
			
		dlg.close()
				
	#
	# retrieves new data and updates the list.
	#
	def update_list(self, data, content_list):
			
		
		"""Updates the list widget with new data."""

		# Either an error dialog has been shown, or the user
		# aborted the download. Anyway, there's no new data
		# to put in the list, so lets just keep the old.
		if data == None:
			return False

		self.data = data		
		self.listItems = []		
		
		if (self.settings[1] == 1 and self.enableDlg):
			dlg = xbmcgui.DialogProgress()
			dlg.create('YouTube', 'Downloading...')

		self.listClear(content_list)
		
		counter = 0
		totalsize = len(self.data)		

		for desc, id in self.data:
			
			author = ""
			out2 = ""
			out = str(desc)
			path = ""

			#set image													
			if (self.settings[1] == 1 and self.yt.thumbnail is not None and len(self.yt.thumbnail) > counter):
				
				percent = int((counter * 100.0) / totalsize)
				if (self.enableDlg):
					dlg.update(percent)				
                
				thumbdet = self.yt.thumbnail[counter]
				try:

					# get the id out of the image, to use as a key for storing the thumbnail.
					mypattern = re.compile('vi/([^/]+)/')
					match = mypattern.search(thumbdet)
					if (match is not None):
						imgName = 'thumb' + str(match.group(1)) + '.jpg'
					else:
						imgName = 'thumb' + str(counter) + '.jpg'
											
					path = os.path.join(self.base_path, 'data', imgName)
					
					if (not os.path.exists(path)):						
						thumb = self.yt.retrieve(thumbdet, None, {} , 0, False)					
						fp = open(path, 'wb')				
						fp.write(thumb)
						fp.close()				
					
				except:
					pass
					


			out2 = ""
			out3 = ""

			if (self.yt.duration is not None and len(self.yt.duration) > counter):
				d = int(self.yt.duration[counter])
				t = str(d % 60)
				duration = str(str(d / 60) + ":" + t)				
				out2 = str(duration) + ""
			
			if (self.yt.author is not None and len(self.yt.author) > counter):
				author = self.yt.author[counter]
				out2 += "\n" + str(author[:15])
				
			if (self.yt.views is not None and len(self.yt.views) > counter):
				views = self.yt.views[counter]	
				out2 += "\n" + str(views) + " views"
				
			if (self.yt.rating is not None and len(self.yt.rating) > counter):
				rating = self.yt.rating[counter]					
				out2 += "\n" + str(rating) + " rating"			

			if path == "":
				path = os.path.join(self.base_path, 'data', "noimage.png")
				
			item = xbmcgui.ListItem (label=out, label2=out2, thumbnailImage=path)			
			
			self.listItems.append(item)
			
			#tempPos = content_list.getCurrentListPosition()
			content_list.addItem(item)
			#content_list.setCurrentListPosition(tempPos)

			counter += 1
		

		# if thumbs enabled
		if (self.settings[1] == 1 and self.enableDlg):
			dlg.close()
			#self.enableDlg = False

		return True


	#
	# get list wrapper for control lists vs built in window lists.
	#
	def getList(self):
	
		return self

	#wrapper method in case of built in window list vs control list.	
	def getListPosition(self, ctrl):
		
		return ctrl.getCurrentListPosition()		


	#
	# get videos by a user from current position in list
	#
	def videos_by_user(self):
		
		pos = self.getListPosition(self)		
		user = self.yt.get_author(pos)
		
		if user is not None:			
			self.get_user_videos(self.yt.get_user_videos, 'Videos by', user)

	#
	# get videos by a user from current position in list
	#
	def playlists_by_user(self):
		
		pos = self.getListPosition(self)		
		user = self.yt.get_author(pos)
		
		if user is not None:			
			self.get_user_playlists(self.yt.get_user_playlists, 'Playlists by', user)


	#
	# get videos by a user from current position in list
	#
	def related_videos(self):
		
		pos = self.getListPosition(self)				
		
		if pos is not None:			
			self.get_youtube("Related Videos", pos, self.yt.get_related)

	#
	# get videos by a user from current position in list
	#
	def response_videos(self):
		
		pos = self.getListPosition(self)				
		
		if pos is not None:			
			self.get_youtube("Video Responses", pos, self.yt.get_responses)
	
			

	def play_clip(self, id):
		"""Get the url for the id and start playback."""

		try:
			self.current_position = self.getCurrentListPosition()
			file = self.download_data(id, self.yt.get_video_url)
			self.player.play(str(file))

			
		except youtube.PrivilegeError, e:
			dlg = xbmcgui.Dialog()
			ret = dlg.yesno('YouTube',
			                'You need to be logged in to watch this clip.',
			                'Do you want to login?')

			# If the user wants to login, and login works, then try again
			try:									
				if ret:
					if self.login():
						self.play_clip(id)
					else:
						dlg = xbmcgui.Dialog()
						dlg.ok('YouTube', 'Login failed.')
			except DownloadError, e:
				err_dlg = xbmcgui.Dialog()
				err_dlg.ok('YouTube', 'There was an error.', e.value)
			except DownloadAbort, e:
				# Just fall through as the method shouldn't return anything.
				pass
		except youtube.VideoStreamError, e:
			dlg = xbmcgui.Dialog()
			dlg.ok('YouTube', 'Unable to play the video clip.')
			
	#
	# context select
	#
	def context_menu_video(self):
	
		items = [ 'Previous Feed', 'Next Feed', 'Other Videos By User', 'Related Videos', 'Video Responses', 'User Playlists', 'Change Sort Order']
		         		         	
		self.context_menu_video_handler(xbmcgui.Dialog().select("Perform action:", items))			
			
	#
	# context handler
	#
	def context_menu_video_handler(self, id):
		if id is 2: # 'Other Videos By User'
			self.videos_by_user()
		elif id is 3: # 'Other Videos By User'
			self.related_videos()			
		elif id is 4: # 'Other Videos By User'
			self.response_videos()	
		elif id is 5: # 'Other Videos By User'
			self.playlists_by_user()
		elif id is 6: # 'Playsts by User'
			self.change_sort()				
		elif id is 0:
			self.go_back()
		elif id is 1:
			self.go_forward()


	def on_control_video_list(self, ctrl):
		"""Handle content list events."""
				
		pos = self.getListPosition(ctrl)				

		try:
			desc, id = self.data[pos]
		except IndexError, e:
			# If this happens something is terribly wrong
			self.close()
			return
		except:
			print 'An unknown error occured in default.py'

		if self.list_state is YouTubeGUI.CONTENT_STATE_VIDEO:
			self.play_clip(id)
		elif self.list_state is YouTubeGUI.CONTENT_STATE_SEARCH_HISTORY:
			self.search(desc)
		else:
			self.not_implemented()

	def on_control_main(self, ctrl):
		"""Handle main menu events."""

		if ctrl is self.getControl(10):			
			self.set_button_state(YouTubeGUI.STATE_FEEDS)
		elif ctrl is self.getControl(11):	
			self.set_button_state(YouTubeGUI.STATE_USERS)
		elif ctrl is self.getControl(14):	
			self.set_button_state(YouTubeGUI.STATE_SEARCH)
		elif ctrl is self.getControl(13):	
			self.show_about()

	def on_control_search(self, ctrl):
		"""Handle search menu events."""
		if ctrl is self.getControl(40):
			self.search()
		elif ctrl is self.getControl(41):
			self.search_history()

	def on_control_users(self, ctrl):
		"""Handle user menu events."""
		if ctrl is self.getControl(90):
			self.get_user_videos(self.yt.get_user_favorites, 'Favorites of', None, str(self.settings[2]))
		elif ctrl is self.getControl(91):
			self.get_user_videos(self.yt.get_user_videos, 'Videos by', None, str(self.settings[3]))
		elif ctrl is self.getControl(92):
			self.get_user_playlists(self.yt.get_user_playlists, 'Playlists of', None, str(self.settings[4]))

	def on_control_feeds(self, ctrl):
		"""Handle feeds menu events."""

		if ctrl is self.getControl(20):			
			self.get_feed('most_recent')
		elif ctrl is self.getControl(21):
			self.get_feed('recently_featured')
		elif ctrl is self.getControl(22):
			self.set_button_state(YouTubeGUI.STATE_TOP_FAVORITES)
		elif ctrl is self.getControl(23):
			self.set_button_state(YouTubeGUI.STATE_TOP_RATED)
		elif ctrl is self.getControl(24):
			self.set_button_state(YouTubeGUI.STATE_MOST_VIEWED)
		elif ctrl is self.getControl(25):
			self.set_button_state(YouTubeGUI.STATE_MOST_DISCUSSED)
		elif ctrl is self.getControl(26):
			self.set_button_state(YouTubeGUI.STATE_MOST_LINKED)
		elif ctrl is self.getControl(27):
			self.set_button_state(YouTubeGUI.STATE_MOST_POPULAR)			
		elif ctrl is self.getControl(28):
		
			cats = self.yt.get_categories()
			
			choice = xbmcgui.Dialog().select("Choose a category:", cats)			
			
			self.get_cat('%s' % cats[choice])


	def not_implemented(event):
		pass

	def get_feed_by_time(self, ctrl, feedstr):
	
		if ctrl is self.getControl(30):
			self.get_feed('%s?time=today' % feedstr)
		elif ctrl is self.getControl(31):
			self.get_feed('%s?time=this_week' % feedstr)
		elif ctrl is self.getControl(32):
			self.get_feed('%s?time=this_month' % feedstr)
		elif ctrl is self.getControl(33):
			self.get_feed('%s?' % feedstr)
		


		
	def init_button_state(self):
		
		xbmcgui.lock()
		
		visible = True		
		self.getControl(10).setVisible(visible)
		self.getControl(11).setVisible(visible)
		self.getControl(14).setVisible(visible)
		self.getControl(13).setVisible(visible)	
		
		visible = False
		self.getControl(02).setVisible(visible)
		self.getControl(06).setVisible(visible)
		self.getControl(20).setVisible(visible)
		self.getControl(21).setVisible(visible)
		self.getControl(22).setVisible(visible)
		self.getControl(23).setVisible(visible)
		self.getControl(24).setVisible(visible)
		self.getControl(25).setVisible(visible)
		self.getControl(26).setVisible(visible)
		self.getControl(27).setVisible(visible)		
		self.getControl(28).setVisible(visible)		

		self.getControl(30).setVisible(visible)
		self.getControl(31).setVisible(visible)
		self.getControl(32).setVisible(visible)
		self.getControl(33).setVisible(visible)

		self.getControl(90).setVisible(visible)
		self.getControl(91).setVisible(visible)
		self.getControl(92).setVisible(visible)
		
		self.getControl(40).setVisible(visible)
		self.getControl(41).setVisible(visible)		

		xbmcgui.unlock()
		
	
	def set_button_state(self, state):
		"""Update button visibility, current state and focused widget."""

		

		# Are we in the main menu?
		visible = bool(state & YouTubeGUI.STATE_MAIN)
		
		self.getControl(10).setVisible(visible)
		self.getControl(11).setVisible(visible)
		self.getControl(14).setVisible(visible)
		self.getControl(13).setVisible(visible)
		self.getControl(15).setVisible(visible)

		if visible:
			dominant = self.getControl(10)

		# Are we in the feeds menu?
		visible = bool(state & YouTubeGUI.STATE_FEEDS)		
		
		
		self.getControl(20).setVisible(visible)
		self.getControl(21).setVisible(visible)
		self.getControl(22).setVisible(visible)
		self.getControl(23).setVisible(visible)
		self.getControl(24).setVisible(visible)
		self.getControl(25).setVisible(visible)
		self.getControl(26).setVisible(visible)
		self.getControl(27).setVisible(visible)
		self.getControl(28).setVisible(visible)
		
		if visible:
			dominant = self.getControl(20)

		# Are we in the most discussed menum (or the most viewed menu)
		visible = bool(state & ~YouTubeGUI.STATE_FEEDS &
		               YouTubeGUI.STATE_MOST_DISCUSSED)

		self.getControl(30).setVisible(visible)
		self.getControl(31).setVisible(visible)
		self.getControl(32).setVisible(visible)
		self.getControl(33).setVisible(visible)

		if visible:
			dominant = self.getControl(30)

		# Are we in the users menu?
		visible = bool(state & YouTubeGUI.STATE_USERS)		

		self.getControl(90).setVisible(visible)
		self.getControl(91).setVisible(visible)
		self.getControl(92).setVisible(visible)

		if visible:
			dominant = self.getControl(90)
		
		# Are we in the search menu?
		visible = bool(state & YouTubeGUI.STATE_SEARCH)

		self.getControl(40).setVisible(visible)
		self.getControl(41).setVisible(visible)

		if visible:
			dominant = self.getControl(40)


		# Set focus to the top-most relevant button, and move
		# to that when leaving the list.
						
		self.setFocus(dominant)
		#self.getControl(50).controlLeft(dominant)
		
		self.state = state

		

base_path = os.getcwd()		

w = YouTubeGUI("youtube.xml", base_path, "DefaultSkin")
w.doModal()
del w
